using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.OleDb;
using System.Data.Odbc;

namespace GenericDataAcces
{
	/// <summary>
	/// Summary description for GenericDataAccess.
	/// </summary>
	public class GenericDataAccess
	{
		//
		// Constantes para definir el tipo de proveedor a utilizar
		//
		public  const string PROVIDER_SQLCLIENT = "Provider_SqlClient";
		public  const string PROVIDER_OLEDB = "Provider_OleDb";
		public  const string PROVIDER_ODBC = "Provider_Odbc";
		//
		private static string _sqlQuery = string.Empty;
		private static string _ConnectionString = string.Empty;
		//
		public static string SetConnectionString {
			get { 
				return _ConnectionString;
			}
			set {
				_ConnectionString = value;
			}
		}
		//
		public static string SetSqlQuery {
			get { 
				return _sqlQuery;
			}
			set {
				_sqlQuery = value;
			}
		}
		//
		public GenericDataAccess()
		{
		}
		// Metodo privato para abrir una conexin
		//
		private static IDbConnection GetConnection(string providerType) {
			IDbConnection objectConnection;

			switch (providerType) {
				case "Provider_SqlClient":
					objectConnection = new SqlConnection(_ConnectionString);
					break;
				case "Provider_OleDb":
					objectConnection = new OleDbConnection(_ConnectionString);
					break;
				case "Provider_Odbc":
					objectConnection = new OdbcConnection(_ConnectionString);
					break;
				default:
					throw(new Exception("Tipo invalido de proveedor"));
			}

			objectConnection.Open();
					
			return (objectConnection);
		}
		//
		private static IDbCommand CreateCommand(string providerType) {
			IDbCommand  objectCmd;
			switch (providerType) {
				case "Provider_SqlClient":
					objectCmd = new SqlCommand();
					break;
				case "Provider_OleDb":
					objectCmd = new OleDbCommand();
					break;
				case "Provider_Odbc":
					objectCmd = new OdbcCommand();
					break;
				default:
					throw(new Exception("Invalid provider type"));
			}

			return (objectCmd);
		}
		//
		public static DataSet ExecuteDataset(string providerType) {
			// 
			// Creamos una conexin
			IDbConnection connection = GetConnection(providerType);
			// Creamos un comando
			IDbCommand cmd = CreateCommand(providerType);
			//
			// Creamos un DataAdpater generico y lo asignamos a tipo de proveedor
			//
			IDataAdapter  da;
			switch (providerType) {
				case "Provider_SqlClient":
					da = new  SqlDataAdapter((SqlCommand)cmd);
					break;
				case "Provider_OleDb":
					da = new  OleDbDataAdapter((OleDbCommand)cmd);
					break;
				case "Provider_Odbc":
					da = new  OdbcDataAdapter((OdbcCommand)cmd);
					break;
				default:
					throw(new Exception("Invalid provider type"));
			}
			
			cmd.Connection = connection;
			cmd.CommandType = CommandType.Text;
			cmd.CommandText = _sqlQuery;
			
			DataSet ds = new DataSet();
			
			da.Fill(ds);
			// 
			cmd.Parameters.Clear();
			
			connection.Close();

			return ds;
		}
	}
}
